-----------------------------------------------------------
--|> DATA PROCESSING
--: Functions that help acquire and manage data involving the PlayerDB
-----------------------------------------------------------
local _, NS = ... --> Addon Name, Namespace
local UnitName = UnitName
local GetGuildInfo = GetGuildInfo
local GetTime = GetTime
local UnitCanAttack = UnitCanAttack
local UnitExists = UnitExists
local UnitClass = UnitClass
local UnitRace = UnitRace
local UnitIsDeadOrGhost = UnitIsDeadOrGhost
local UnitOnTaxi = UnitOnTaxi
local UnitIsPlayer = UnitIsPlayer
local GetUnitName = GetUnitName
local UnitLevel = UnitLevel

--> UpdatePlayerActiveCache
--: Updates data in the PlayerActiveCache and then calls
--: Sends data off the Lists to be processed

function NS.UpdatePlayerActiveCache(name, stealth, dead, role, GUID)
  --> Verify GUID exists
  if not GUID then
    return
  end
  local newPlayerOnList = true
  --: Check for player already in cache
  if NS.PlayerActiveCache[GUID] then
    newPlayerOnList = false
  else
    NS.PlayerActiveCache[GUID] = {}
  end
  --: Update PlayerCache info
  if newPlayerOnList then
    NS.PlayerActiveCache[GUID].GUID = GUID
    --: NAME
    if name then
      NS.PlayerActiveCache[GUID].Name = name
    elseif NS.PlayerDB[name].Name then
      NS.PlayerActiveCache[GUID].Name = NS.PlayerDB[name].Name
    end
    --: LEVEL
    if NS.PlayerDB[name].Level then
      NS.PlayerActiveCache[GUID].Level = NS.PlayerDB[name].Level
      NS.PlayerActiveCache[GUID].Estimated = NS.PlayerDB[name].Estimated
    else -- no level found
      NS.PlayerActiveCache[GUID].Level = 0
      NS.PlayerActiveCache[GUID].Estimated = true
      NS.PlayerDB[name].Level = 0
      NS.PlayerDB[name].Estimated = true
    end
    --: CLASS
    if NS.PlayerDB[name].Class then
      NS.PlayerActiveCache[GUID].Class = NS.PlayerDB[name].Class
    end
    --: GUILD
    if NS.PlayerDB[name].Guild then
      NS.PlayerActiveCache[GUID].Guild = NS.PlayerDB[name].Guild
    end
    --: STEALTH
    if stealth ~= nil then
      NS.PlayerActiveCache[GUID].Stealth = stealth
    end
    --: DEAD
    if dead ~= nil then
      NS.PlayerActiveCache[GUID].Dead = dead
      if dead then
        NS.PlayerActiveCache[GUID].Health = 0
      elseif dead == false and NS.PlayerActiveCache[GUID].Health == 0 then
        NS.PlayerActiveCache[GUID].Health = 1
      end
    end
    --: ROLE
    if role ~= nil then
      NS.PlayerActiveCache[GUID].Role = role
      NS.PlayerDB[name].Role = role
    end
  end
  NS.AddPlayerDataToNearby(GUID, newPlayerOnList)
end

--> Static Role Assignment
function NS.ClassRoleAssign(class)
  if class == "ROGUE" or class == "MAGE" or class == "WARLOCK" or class == "HUNTER" then
    return "DAMAGER"
  end
  return nil
end

--> GET UNIT DATA
local unitUpdateThreshold = 120 -- 2 minutes between guild/race/level/role checks
local gudName
local gudGUID
function NS.GetUnitData(unit)
  if (not unit) or (not UnitExists(unit)) or (not NS.IsUnitValidForTracking(unit)) then
    return -- no valid unit to process
  end
  gudName = NS.GetUnitName(unit) or nil
  gudGUID = UnitGUID(unit) or nil
  if gudName and gudGUID then
    --: Add player to DB if not found
    if not NS.PlayerDB[gudName] then
      NS.PlayerDB[gudName] = {}
      NS.PlayerDB[gudName].Name = gudName
      NS.PlayerDB[gudName].Timestamp = 0
      _, NS.PlayerDB[gudName].Class = UnitClass(unit)
      NS.PlayerDB[gudName].Role = NS.ClassRoleAssign(NS.PlayerDB[gudName].Class)
    end

    --: Update player info if estimated or past update threshold
    if NS.PlayerDB[gudName].Timestamp + unitUpdateThreshold > GetTime() or NS.PlayerDB[gudName].Estimated then
      NS.PlayerDB[gudName].Guild = GetGuildInfo(unit)
      NS.PlayerDB[gudName].Level = UnitLevel(unit)
      NS.PlayerDB[gudName].Race = UnitRace(unit)
    end
    --: Player On Bars?
    if NS.PlayersOnBars[gudGUID] then
      NS.PlayerActiveCache[gudGUID].OnTaxi = UnitOnTaxi(unit) or nil
      NS.UnitHealthCheck(unit)
    end
    NS.PlayerDB[gudName].Estimated = false
    NS.PlayerDB[gudName].Timestamp = GetTime()
    NS.UpdatePlayerActiveCache(gudName, nil, nil, nil, gudGUID) -- (name, stealth, dead, role, GUID)
  end
  gudName, gudGUID = nil, nil
end

--> Is Unit Valid For Tracking
local iuvGUID
function NS.IsUnitValidForTracking(unit)
  if (not unit) or (not UnitIsPlayer(unit)) then -- input check
    return false
  end
  if GetUnitName(unit) == "Unknown" then
    return false
  end
  if (UnitCanAttack("player", unit) or UnitIsEnemy("player", unit) or NS.Options.DEBUG) then -- enemy player check
    return true
  else
    if not NS.Options.DEBUG and NS.PlayerActiveCache[UnitGUID(unit)] then
      iuvGUID = UnitGUID(unit) or nil
      NS.PlayerActiveCache[iuvGUID] = nil
      if NS.ActiveList[iuvGUID] then
        NS.ActiveList[iuvGUID].TimeAdded = 0
        NS.ActiveList[iuvGUID].TimeUpdated = 0
      end
      if NS.ActiveDeadList[iuvGUID] then
        NS.ActiveDeadList[iuvGUID].TimeAdded = 0
        NS.ActiveDeadList[iuvGUID].TimeUpdated = 0
      end
      if NS.InactiveList[iuvGUID] then
        NS.InactiveList[iuvGUID].TimeAdded = 0
        NS.InactiveList[iuvGUID].TimeUpdated = 0
      end
      if NS.InactiveDeadList[iuvGUID] then
        NS.InactiveDeadList[iuvGUID].TimeAdded = 0
        NS.InactiveDeadList[iuvGUID].TimeUpdated = 0
      end
      NS.CurrentNameplates[iuvGUID] = nil
      if NS.NearbyCount == 1 then
        NS.CoreUI.Initialize()
      end
      NS.ManageListTimeouts()
    end
  end
  return false
end

--> weizPVP: GetUnitName()
--: @returns "playerName-realmName" for all players, no matter the realm
local name, realm
function NS.GetUnitName(unit)
  if not unit then -- check for unit
    return
  end
  name, realm = UnitName(unit, true)
  if not name then
    return
  end
  if not realm then --: same realm
    return name .. "-" .. NS.PlayerRealm
  else --: different realm
    realm = gsub(realm, "[%s%-]", "")
    return name .. "-" .. realm
  end
end

--> UnitHealthCheck
local UnitHealth = UnitHealth
local UnitHealthMax = UnitHealthMax
local UnitGUID = UnitGUID
local GUID
function NS.UnitHealthCheck(unit)
  GUID = UnitGUID(unit) or nil
  if GUID and UnitExists(unit) then
    NS.PlayerActiveCache[GUID].Dead = UnitIsDeadOrGhost(unit)
    NS.PlayerActiveCache[GUID].Health = UnitHealth(unit) / UnitHealthMax(unit) * 100
    NS.RefreshBarByGUID(GUID)
  end
end
